package org.gcube.portlets.user.geoportaldataentry.client.ui.edit;

import java.util.List;

import org.gcube.application.geoportalcommon.shared.products.BaseConcessioneDV;
import org.gcube.application.geoportalcommon.shared.products.ConcessioneDV;
import org.gcube.application.geoportalcommon.shared.products.content.WorkspaceContentDV;
import org.gcube.application.geoportalcommon.shared.products.model.AbstractRelazioneScavoDV;
import org.gcube.application.geoportalcommon.shared.products.model.LayerConcessioneDV;
import org.gcube.application.geoportalcommon.shared.products.model.RelazioneScavoDV;
import org.gcube.application.geoportalcommon.shared.products.model.UploadedImageDV;
import org.gcube.portlets.user.geoportaldataentry.client.ConstantsGeoPortalDataEntryApp.RECORD_TYPE;
import org.gcube.portlets.user.geoportaldataentry.client.GeoPortalDataEntryApp;
import org.gcube.portlets.widgets.mpformbuilder.client.ui.upload.MultipleDilaogUpload;

import com.github.gwtbootstrap.client.ui.Button;
import com.github.gwtbootstrap.client.ui.ControlGroup;
import com.github.gwtbootstrap.client.ui.Controls;
import com.github.gwtbootstrap.client.ui.Label;
import com.github.gwtbootstrap.client.ui.ListBox;
import com.github.gwtbootstrap.client.ui.constants.LabelType;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HTMLPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * The Class UpdateFileset.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR francesco.mangiacrapa@isti.cnr.it
 * 
 *         Sep 27, 2021
 */
public class UpdateFileset extends Composite {

	private static final String _700PX = "700px";

	private static final String UPLOAD_MISSING_FILE = "You must upload a file";

	private static UpdateFilesetUiBinder uiBinder = GWT.create(UpdateFilesetUiBinder.class);

	/**
	 * The Interface UpdateFilesetUiBinder.
	 *
	 * @author Francesco Mangiacrapa at ISTI-CNR francesco.mangiacrapa@isti.cnr.it
	 * 
	 *         Sep 27, 2021
	 */
	interface UpdateFilesetUiBinder extends UiBinder<Widget, UpdateFileset> {
	}

	@UiField
	ListBox listBoxPaths;

	@UiField
	ControlGroup cgSelectFile;

	@UiField
	Controls controlsContent;

	@UiField
	HTMLPanel uploadFileContainer;

	@UiField
	Button buttonUpdate;

	private List<String> listFileSetPaths;

	private boolean placeholderRemoved = false;

	private String recordId;

	private BaseConcessioneDV selectedConcessione;

	private ConcessioneDV fullConcessione;

	private RECORD_TYPE recordType;

	private MultipleDilaogUpload mDU;

	private ChangeHandler listBoxIndexHandler;

	/**
	 * Instantiates a new update fileset.
	 *
	 * @param listFileSetPaths the list file set paths
	 */
	public UpdateFileset(BaseConcessioneDV selectedConcessione, RECORD_TYPE recordType, List<String> listFileSetPaths) {
		initWidget(uiBinder.createAndBindUi(this));
		this.selectedConcessione = selectedConcessione;
		this.recordType = recordType;
		this.listFileSetPaths = listFileSetPaths;
		listBoxPaths.addItem("Select a section...");
		for (String path : listFileSetPaths) {
			listBoxPaths.addItem(path);
		}
		listBoxPaths.setWidth(_700PX);

		// add handler on select
		listBoxPaths.addChangeHandler(new ChangeHandler() {

			@Override
			public void onChange(ChangeEvent event) {
				GWT.log("Profile type selection changed...");
				cgSelectFile.setVisible(false);

				if (!placeholderRemoved) {
					listBoxPaths.removeItem(0); // this is the placeholder
					placeholderRemoved = true;
				}
				showUploadFileGUI();
			}

		});

		GeoPortalDataEntryApp.geoportalDataEntryService.getRecord(selectedConcessione.getItemId(), recordType,
				new AsyncCallback<ConcessioneDV>() {

					@Override
					public void onSuccess(ConcessioneDV theRecord) {
						fullConcessione = theRecord;
					}

					@Override
					public void onFailure(Throwable caught) {
						Window.alert(caught.getMessage());
					}
				});

		buttonUpdate.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent event) {

				if (mDU != null) {
					if (checkValidUpload()) {
						Window.alert("I can save...");
					} else {
						Window.alert("New file not found. " + UPLOAD_MISSING_FILE);
					}
				}

			}
		});

	}

	/**
	 * Show upload file GUI.
	 */
	private void showUploadFileGUI() {
		uploadFileContainer.setVisible(true);
		buttonUpdate.setVisible(false);
		uploadFileContainer.clear();
		controlsContent.clear();

//		listBoxIndex.clear();
		cgSelectFile.setVisible(true);

		ListBox listBoxIndex = new ListBox();
		listBoxIndex.setWidth(_700PX);
		listBoxIndex.addItem("Select a content...");

		if (listBoxPaths.getSelectedItemText().contains("abstract_relazione")) {
			AbstractRelazioneScavoDV ar = fullConcessione.getAbstractRelazioneScavo();
			if (ar == null) {
				showMessage("abstract_relazione" + " NOT AVAILABLE", LabelType.WARNING);
				return;
			}

			int posizIndex = 0;
			fillListBoxToBeReplaced(listBoxIndex, "abstract_relazione", posizIndex, ar.getTitolo(),
					ar.getListWsContent());

		} else if (listBoxPaths.getSelectedItemText().contains("immagini")) {
			List<UploadedImageDV> listImmagini = fullConcessione.getImmaginiRappresentative();
			if (listImmagini == null || listImmagini.isEmpty()) {
				showMessage("immagini" + " NOT AVAILABLE", LabelType.WARNING);
				return;
			}
			int posizIndex = 0;
			for (UploadedImageDV uploadedImageDV : listImmagini) {
				fillListBoxToBeReplaced(listBoxIndex, "immagini", posizIndex, uploadedImageDV.getTitolo(),
						uploadedImageDV.getListWsContent());
				posizIndex++;
			}

		} else if (listBoxPaths.getSelectedItemText().contains("relazione")) {
			RelazioneScavoDV realzione = fullConcessione.getRelazioneScavo();
			if (realzione == null || realzione.getListWsContent() == null) {
				showMessage("relazione" + " NOT AVAILABLE", LabelType.WARNING);
				return;
			}
			int posizIndex = 0;
			fillListBoxToBeReplaced(listBoxIndex, "relazione", posizIndex, realzione.getTitolo(),
					realzione.getListWsContent());

		} else if (listBoxPaths.getSelectedItemText().contains("posizionamentoScavo")) {
			LayerConcessioneDV posiz = fullConcessione.getPosizionamentoScavo();
			if (posiz == null || posiz.getListWsContent() == null || posiz.getListWsContent().isEmpty()) {
				showMessage("posizionamentoScavo" + " NOT AVAILABLE", LabelType.WARNING);
				return;
			}
			int posizIndex = 0;
			fillListBoxToBeReplaced(listBoxIndex, "posizionamentoScavo", posizIndex, posiz.getTitolo(),
					posiz.getListWsContent());

		} else if (listBoxPaths.getSelectedItemText().contains("piante")) {
			List<LayerConcessioneDV> piante = fullConcessione.getPianteFineScavo();
			if (piante == null || piante.isEmpty()) {
				showMessage("piante" + " NOT AVAILABLE", LabelType.WARNING);
				return;
			}
			int posizIndex = 0;
			for (LayerConcessioneDV lcDV : piante) {
				fillListBoxToBeReplaced(listBoxIndex, "piante", posizIndex, lcDV.getTitolo(), lcDV.getListWsContent());
				posizIndex++;
			}

		}

		controlsContent.add(listBoxIndex);

//		if (listBoxIndex.getItemCount() > 1) {
//			listBoxIndex.addChangeHandler(new ChangeHandler() {
//
//				@Override
//				public void onChange(ChangeEvent event) {
//					showFileBrowseInteraction();
//				}
//			});
//		}

	}

	private void fillListBoxToBeReplaced(ListBox listBoxIndex, String section, int posizIndex, String title,
			List<WorkspaceContentDV> listWSC) {

//		ListBox listBoxIndex = new ListBox();
//		listBoxIndex.addItem("Select a content...");

		if (listWSC == null || listWSC.isEmpty()) {
			showMessage(section + " DOES NOT CONTAIN FILES", LabelType.WARNING);
			return;
		}

		listBoxIndex.addItem(title, posizIndex + "");

		// adding handler once
		if (posizIndex == 0) {
			listBoxIndex.addChangeHandler(new ChangeHandler() {

				@Override
				public void onChange(ChangeEvent event) {
					showFileBrowseInteraction();
				}
			});
		}
//		
//		controlsContent.add(listBoxIndex);

		for (WorkspaceContentDV wsContent : listWSC) {
			String name = wsContent.getName() == null || wsContent.getName().isEmpty() ? wsContent.getLink()
					: wsContent.getName();
		}
	}

	private void showFileBrowseInteraction() {
		uploadFileContainer.clear();
		HTML label = new HTML();
		label.setHTML("With new one:");

		uploadFileContainer.add(label);
		mDU = new MultipleDilaogUpload();
		uploadFileContainer.add(mDU);
		buttonUpdate.setVisible(true);
	}

	private boolean checkValidUpload() {
		GWT.log("Checking is valid File");

		if (mDU.getFileUploadingState() != null && mDU.getFileUploadingState().getFile() != null) {
			return mDU.getFileUploadingState().getFile().getTempSystemPath() != null;
		}

		return false;
	}

	private void showMessage(String txt, LabelType type) {
		Label l = new Label();
		l.setType(type);
		l.setText(txt);
		uploadFileContainer.add(l);
	}

}
